# -*- coding: utf-8 -*-
# --------------------------------------------------------------------------------- #
# YT-Scripts
# Extracts a subregion from a Simulation checkpoint with a defined grid
#
# Marcial Becerril, @ 18 April 2021
# Latest Revision: 18 Apr 2021, 20:30 GMT-6
#
# For all kind of problems, requests of enhancements and bug reports, please
# write to me at:
#
# mbecerrilt92@gmail.com
# mbecerrilt@inaoep.mx
#
# --------------------------------------------------------------------------------- #

import yt
import time
import numpy as np
from matplotlib import *


def extract_subregion(ds, param, region, grid_res, save=True, *args, **kwargs):
    """
        Extract a subregion with a defined grid from a simulated data cube
        Parameters
        ----------
        ds : YTobject
        region : array
        	Limits of the subregion to extract:
        	[[xmin, ymin, zmin], [xmax, ymax, zmax]] 
        grid_res : array
            Defined resolution
            [xgrid, ygrid, zgrid]
        save : boolean
        	Save figure? .fits as default
        ----------
    """
    
    # Get the format
    fmt_sv = kwargs.pop('fmt', 'npy')
    # Name file
    default_name = ds.basename
    name = kwargs.pop('name', default_name)
    # Get units
    units = kwargs.pop('units', 'g*cm**-3')
    # Verbose
    verbose = kwargs.pop('verbose', None)

    # Get the subregion
    if region == "all":
        sub_reg = ds.r[::1j*grid_res[0], ::1j*grid_res[1], ::1j*grid_res[2]]
    else:
        sub_reg = ds.r[region[0][0]:region[0][1]:1j*grid_res[0], region[1][0]:region[1][1]:1j*grid_res[1],
                       region[2][0]:region[2][1]:1j*grid_res[2]]

    start_time = time.time()
    buff_img = sub_reg[param][:,:,:].to(units).d
    end_time = time.time()

    if verbose:
        print ("Extracting time %.3f seconds" % (end_time - start_time))

    # Save the map
    if save:
        if fmt_sv == 'fits':
            # Save data as fits file
            # TODO [Under construction]
            # ---------------------------------------------
            # Get the header
            prj_fits = yt.FITSSlice(ds, 'x', [units])
            header = prj_fits[units].header

            hdu = fits.PrimaryHDU(buff_img, header=header)
            os.system("rm -f " + name + ".fits")
            hdu.writeto(name + ".fits")
            # ---------------------------------------------

        elif fmt_sv == 'hdf5':
            # Save data as hdf5 file
            print ("Option not available yet")

        elif fmt_sv == 'npy':
            # Save as npy file
            np.save(name, buff_img)

    return buff_img

pc = 3.08567758e18 
delta   = 1.0 * pc # --> L_subbox = delta
xc_zoom = 0.0 * pc # Center coords of the sub box
yc_zoom = 0.0 * pc
zc_zoom = 0.0 * pc

ranges = [ [xc_zoom - 0.5 * delta, xc_zoom + 0.5 * delta],
		  [yc_zoom - 0.5 * delta, yc_zoom + 0.5 * delta],
		  [zc_zoom - 0.5 * delta, zc_zoom + 0.5 * delta] ]

# Example
path_example = "/mnt/zfs-pool/home/proys.manuel.zamora/karina/plotcolli/flows_hdf5_plt_cnt_0141"
# Parameter
param = 'dens'
# Subregion
#sub_reg = 'all'
sub_reg = ranges
# Grid resolution
res = [128, 128, 128]
# Read the image
ds = yt.load(path_example)

buff_img = extract_subregion(ds, param, sub_reg, res)

